/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvworker.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.
 
#include <QDebug>
#include <QMutex>
#include <QWaitCondition>
#include <QApplication>

#include <qvcore/qvworker.h>

#include <qvgui/qvimagecanvas.h>

QVWorker::QVWorker(QString name):QVPropertyHolder(name),
	cpuStat(), numIterations(0), status(Running), inited(false), initedParam(), errorStatus(), cameras()
	{
	qDebug() << "QVWorker::QVWorker(" << name << ")";
	Q_ASSERT_X(qvApp != NULL, "QVWorker::QVWorker()", "QVApplication doesn't exists");
	addProperty<bool>("print stats", inputFlag, false, "Enables realtime stats console output for worker");
	cpuStat.printStatsToConsole(getPropertyValue<bool>("print stats"));
	qDebug() << "QVWorker::QVWorker(" << name << ") <- return";
	};
			
QVWorker::~QVWorker()
	{
	qDebug() << "QVWorker::~QVWorker()";
	qDebug() << "QVWorker::~QVWorker() <- return";
	}

//PEDROE:
void QVWorker::run()
	{
	qDebug() << "QVWorker::run()";

	while(1) {
		iterate();
		if(status == Finished) break;
	}
	unlink();

	qDebug() << "QVWorker::run() <- return";
	}


void QVWorker::iterate()
	{
	qDebug() << "QVWorker::iterate()";
	qDebug() << "QVWorker::iterate(): iteration" << numIterations;

	// Avoids "apparent hanging" (greedy ocupation of CPU by extremely fast
	// workers, such as paused ones).
	//usleep(100);

	cpuStat.step();

	switch (status)
		{
		case RunningOneStep:
			qDebug() << "QVWorker::iterate(): RunningOneStep";
			status = Paused;

		case Running:
			{
			emit startIteration();
			timeFlag("System");
			readInputProperties();
			worker();
			writeOutputProperties();
			numIterations++;
			emit endIteration();
			break;
			}

		case Paused:
			qDebug() << "QVWorker::iterate(): Paused";
			{
			/*QMutex mutex;
			QWaitCondition condition;
			qDebug() << "QVWorker::iterate(): locking";
			mutex.lock();
			qDebug() << "QVWorker::iterate(): pausing 100 milisecs";
			condition.wait(&mutex,100);
			qDebug() << "QVWorker::iterate(): unlocking";
			mutex.unlock();*/
			}
			break;
					
		case Finished:
			qDebug() << "QVWorker::iterate(): Finished";
			break;
		}

	qDebug() << "QVWorker::iterate() <- return";
	}
	
	/* Si necesitamos procesar eventos en QVWorker, hacemos algo así como esto en QWorker::run():
	QTimer timer;
	timer.start(0);
	connect(timer, SIGNAL(timeout()),this,SLOT(myquit()));
	while(1) {
		iterate();
		exec();
		//PEDROE if stopped disconnect(timer, SIGNAL(timeout()),this,SLOT(myquit())); BLA BLA
	}
	*/
