/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvcamerawidget.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvgui/qvcamerawidget.h>

QVCameraWidget::QVCameraWidget(QVCamera *cam, QWidget *parent):QWidget(parent)
	{
	setupUi(this);
	this->camera = cam;
	this->sliderActive = true;
	if (this->camera->isLiveCamera())
		seekSlider->setEnabled(false);

	// Connection of signals in camera control widget to private slots...
	/*
	connect(pause_button,SIGNAL(pressed()),this,SLOT(pausePressed()));
	connect(play_button,SIGNAL(pressed()),this,SLOT(unpausePressed()));
	connect(next_button,SIGNAL(pressed()),this,SLOT(nextFramePressed()));
	connect(stop_button, SIGNAL(pressed()),this,SLOT(closePressed()));

	connect(this,SIGNAL(pause_pressed()),camera,SLOT(pauseCam()));
	connect(this,SIGNAL(unpause_pressed()),camera,SLOT(unpauseCam()));
	connect(this,SIGNAL(next_frame_pressed()),camera,SLOT(nextFrameCam()));
	connect(this,SIGNAL(close_pressed()),camera,SLOT(closeCam()));
	*/

	connect(pause_button,SIGNAL(pressed()),camera,SLOT(pauseCam()));
	connect(play_button,SIGNAL(pressed()),camera,SLOT(unpauseCam()));
	connect(next_button,SIGNAL(pressed()),camera,SLOT(nextFrameCam()));
	connect(stop_button, SIGNAL(pressed()),camera,SLOT(closeCam()));

	/////////////////////////////////////
	connect(seekSlider,SIGNAL(sliderPressed()),this,SLOT(seekPressed()));
	connect(seekSlider,SIGNAL(sliderReleased()),this,SLOT(seekReleased()));
	connect(seekSlider,SIGNAL(valueChanged(int)),this,SLOT(seekMoved()));
	connect(seekSlider,SIGNAL(sliderMoved(int)),this,SLOT(seekMoved()));

	connect(this,SIGNAL(seek_pressed(QVCamera::TSeekType,double)), camera,SLOT(seekCam(QVCamera::TSeekType,double)));

	// ...and from private signals to public camera slots...
	connect(speed_button,SIGNAL(pressed()),this,SLOT(speedPressed()));
	connect(this,SIGNAL(speed_pressed(double)), camera,SLOT(setSpeedCam(double)));

	// ... and, finally, of camera newGrab and newRead signals to the
	// updateVisibleInfo slot...
	connect(camera,SIGNAL(newRead()), this, SLOT(updateCameraVisibleInfoSlot()));
	connect(camera,SIGNAL(newGrab()), this, SLOT(updateCameraVisibleInfoSlot()));
	connect(camera,SIGNAL(statusChange(QVCamera::TCameraStatus)),
		this,SLOT(updateCameraStateSlot(QVCamera::TCameraStatus)));

	// ... and the closed signal of the camera, also to the closePressed slot:
	//connect(camera,SIGNAL(camClosed()),this,SLOT(closePressed()));
	}

void QVCameraWidget::seekPressed()
	{
	sliderActive = false;
	}

void QVCameraWidget::seekReleased()
	{
	sliderActive = true;
	emit seek_pressed((QVCamera::TSeekType)1,seekSlider->value());
	}

void QVCameraWidget::seekMoved()
	{
	//emit seek_pressed((QVCamera::TSeekType)1,seekSlider->value());
	}

void QVCameraWidget::speedPressed()
	{
	emit speed_pressed(speed_spin_box->value());
	}

// Public slots:
void QVCameraWidget::updateCameraVisibleInfoSlot()
{
    // We update the fixed camera widget visible info (file name, fps, speed 
    // and size):
    filename_label->setText(camera->getUrlBase());
    fps_orig_label->setText(QString("fps: %1").
                            arg(static_cast<int>(camera->getFPS())));
    speed_label->setText(QString("speed: x %1")
                         .arg(camera->getSpeed(),0,'f',2));
    size_label->setText(QString("size: %1x%2").
                        arg(camera->getRows()).arg(camera->getCols()));
    // We update the variable camera widget visible info (frames
    // grabbed/updated and elapsed/total time):
    frames_label->setText(QString("frames grabbed/read: %1/%2").
                       arg(camera->getFramesGrabbed()).
                       arg(camera->getFramesRead()));
    pos_label->setText(QString("secs: %1/%2").
                       arg(camera->getTimePos()).
                       arg(camera->getTimeLength()));
    int percen = (int)((100*camera->getTimePos())/camera->getTimeLength());
	if (sliderActive) seekSlider->setValue(percen); 
}

void QVCameraWidget::updateCameraStateSlot(QVCamera::TCameraStatus status)
	{
	switch(status)
		{
		//case QVCamera::EndOfFile:
		case QVCamera::Closed:
			stop_button->setEnabled(FALSE);
			pause_button->setEnabled(FALSE);
			play_button->setEnabled(FALSE);
			next_button->setEnabled(FALSE);
			break;

		case QVCamera::Running:
			pause_button->setEnabled(TRUE);
			play_button->setEnabled(FALSE);
			next_button->setEnabled(FALSE);
			break;

		case QVCamera::RunningOneStep:
		case QVCamera::Paused:
			pause_button->setEnabled(FALSE);
			play_button->setEnabled(TRUE);
			next_button->setEnabled(TRUE);
			break;

		default:
			break;
		}
	}

