/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvparamwidget.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVPARAMWIDGET_H
#define QVPARAMWIDGET_H
#include <qvcore/qvpropertyholder.h>
#include <ui/ui_qvintparamwidget.h>
#include <ui/ui_qvdoubleparamwidget.h>
#include <QtGui/QCheckBox>

/// @class QVIntParamWidget qvparamwidget.h qvgui/qvparamwidget.h
/// @brief QVIntParamWidget class.
///
/// This is a widget that presents a spin box to the user, where he can inspect and modify the integer value of
/// an integer type property, from a QVPropertyHolder object.
/// @ingroup qvgui
class QVIntParamWidget: public QWidget
	{
		Q_OBJECT
	public:
		Ui::QVIntParamWidgetForm form;
	
		/// @brief Creates a new QVIntParamWidget object
		/// @param propertyHolder which contains the param that should be inspected
		/// @param property name of the param that will be inspected
		QVIntParamWidget(QVPropertyHolder * propertyHolder, const QString property, QWidget *parent = 0);
	
	private slots:
		void setValue(int value);
	
	private:
		int min, max;
		QString paramName;
		QVPropertyHolder * propertyHolder;
	};

/// @class QVDoubleParamWidget qvparamwidget.h qvgui/qvparamwidget.h
/// @brief QVDoubleParamWidget class.
///
/// This is a widget that presents a slider to the user, where he can inspect and modify the double value of
/// a double type property, from a QVPropertyHolder object.
class QVDoubleParamWidget: public QWidget
	{
		Q_OBJECT
	
	public:	
		Ui::QVDoubleParamWidgetForm form;
	
		/// @brief Creates a new QVDoubleParamWidget object
		/// @param propertyHolder which contains the param that should be inspected
		/// @param property name of the param that will be inspected
		QVDoubleParamWidget(QVPropertyHolder * propertyHolder, const QString property, QWidget *parent = 0);
	
	private:
		double min, max;
		double fraction;
		QString paramName;
		QVPropertyHolder * propertyHolder;
	
	public slots:
		void updateValue(int value);
	
	private slots:
		void setValue(double value);
	};

/// @class QVBoolParamWidget qvparamwidget.h qvgui/qvparamwidget.h
/// @brief QVBoolParamWidget class.
///
/// This is a widget that presents a check-box to the user, where he can inspect and modify the boolean value of
/// a boolean type property, from a QVPropertyHolder object.
class QVBoolParamWidget: public QCheckBox
	{
		Q_OBJECT
	public:	
	
		/// @brief Creates a new QVBoolParamWidget object
		/// @param propertyHolder which contains the param that should be inspected
		/// @param property name of the param that will be inspected
		QVBoolParamWidget(QVPropertyHolder * propertyHolder, const QString property, QWidget *parent = 0);
	
	public slots:
		void stateChangedSlot(int state);
	
	private slots:
		void setValue(bool value);
	
	private:
		QString paramName;
		QVPropertyHolder * propertyHolder;
	};

#endif

