/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvworkerinterfacesmall.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvgui/qvworkerinterfacesmall.h>

QVWorkerInterfaceSmall::QVWorkerInterfaceSmall(QVWorker *worker, QWidget *parent): QWidget(parent)
	{
	qDebug() << "QVWorkerInterfaceSmall::QVWorkerInterfaceSmall("<< worker->getName() << ")";
	form.setupUi(this);

	setWindowTitle(worker->getName());
	form.groupBox->setTitle(QString("Control: ")+worker->getName());
	this->worker = worker;

	#ifdef PACKAGE_QWT
	this->cpuPlot = new QVCpuPlot(&(this->worker->cpuStat));
	this->cpuPlotSmall = new QVCpuPlot(&(this->worker->cpuStat),false);
	form.vboxLayout->addWidget(this->cpuPlotSmall);
	connect(form.stat_button, SIGNAL(pressed()),cpuPlot,SLOT(show()));
	#else
	// de momento, no se esconde el botón, no se compila :P
	//form.stat_button.setVisible(false);
	#endif

	// Connect worker slots with form's buttons
	connect(form.pause_button,SIGNAL(pressed()),worker,SLOT(pause())/*,Qt::QueuedConnection*/);
	connect(form.play_button,SIGNAL(pressed()),worker,SLOT(unPause())/*,Qt::QueuedConnection*/);
	connect(form.next_button,SIGNAL(pressed()),worker,SLOT(step())/*,Qt::QueuedConnection*/);
	connect(form.stop_button, SIGNAL(pressed()),worker,SLOT(finish())/*,Qt::QueuedConnection*/);

	// Connecting slot with worker update signal
	connect(worker,SIGNAL(endIteration()),this,SLOT(newIterationSlot()),Qt::BlockingQueuedConnection);
	connect(worker,SIGNAL(statusUpdate(QVWorker::TWorkerStatus)),
		this,SLOT(statusUpdate(QVWorker::TWorkerStatus))/*,Qt::QueuedConnection*/);

	show();
	qDebug() << "QVWorkerInterfaceSmall::QVWorkerInterfaceSmall(" << worker->getName() << ") <- return";
	}

void QVWorkerInterfaceSmall::newIterationSlot()
	{ form.iterationsLabel->setText(QString().setNum(worker->getIteration())); }

void QVWorkerInterfaceSmall::statusUpdate(QVWorker::TWorkerStatus status)
	{
	switch(status)
		{
		case QVWorker::Finished:
			form.stop_button->setEnabled(FALSE);
			form.pause_button->setEnabled(FALSE);
			form.play_button->setEnabled(FALSE);
			form.next_button->setEnabled(FALSE);
			break;

		case QVWorker::Running:
			form.pause_button->setEnabled(TRUE);
			form.play_button->setEnabled(FALSE);
			form.next_button->setEnabled(FALSE);
			break;

		case QVWorker::RunningOneStep:
		case QVWorker::Paused:
			form.pause_button->setEnabled(FALSE);
			form.play_button->setEnabled(TRUE);
			form.next_button->setEnabled(TRUE);
			break;

		default:
			break;
		}
	}

